// Copyright (C) 2008
// by all contributors..

// This file is part of SCNSL.

// SCNSL is free software:
// you can redistribute it and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option)
// any later version.

// SCNSL is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY;
// without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.
// See the GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with SCNSL,
// in a file named LICENSE.txt.
// If not, see <http://www.gnu.org/licenses/>.



/// @file
///  Basic data types.



#include "scnsl/core/data_types.hh"

namespace tlm
{

const unsigned int tlm_extension<Scnsl::Core::data_extension_t>::ID = tlm_extension_base::register_extension();

} // tlm


using namespace Scnsl::Core;



// /////////////////////////////////////////////////////////////////////////////
// lobe_t
// /////////////////////////////////////////////////////////////////////////////

lobe_t::lobe_t():
    beamwidth(0),
    gain(0)
{
    direction[0]=0;
    direction[1]=0;
    direction[2]=0;
}

lobe_t::lobe_t(const double d[], const double bw, const double g):
    beamwidth(bw),
    gain(g)
{
    direction[0]=d[0];
    direction[1]=d[1];
    direction[2]=d[2];
}

// /////////////////////////////////////////////////////////////////////////////
// node_properties_t
// /////////////////////////////////////////////////////////////////////////////

node_properties_t::node_properties_t()
    :id(0),
      sourceNode(),
      x(0),
      y(0),
      z(0),
      bitrate(0),
      transmission_power(0),
      receiving_threshold(0),
      lobes()
{
    // ntd
}

node_properties_t::node_properties_t(const node_properties_t & np):
    id(np.id),
    sourceNode(np.sourceNode),
    x(np.x),
    y(np.y),
    z(np.z),
    bitrate(np.bitrate),
    transmission_power(np.transmission_power),
    receiving_threshold(np.receiving_threshold),
    lobes(np.lobes)
{
    // ntd
}

node_properties_t &node_properties_t::operator=(const node_properties_t & np)
{
    if (this == &np) return *this;
    id = np.id;
    x = np.x;
    y = np.y;
    z = np.z;
    bitrate = np.bitrate;
    transmission_power = np.transmission_power;
    receiving_threshold = np.receiving_threshold;
    lobes = np.lobes;
    return *this;
}


// /////////////////////////////////////////////////////////////////////////////
// node_factory_infos_t
// /////////////////////////////////////////////////////////////////////////////

node_factory_infos_t::node_factory_infos_t():
    nodeStepX(0),
    nodeStepY(0),
    nodeStepZ(0),
    nodeNumberX(1),
    nodeNumberY(1),
    nodeNumberZ(1)
{}


// /////////////////////////////////////////////////////////////////////////////
// data_extension_t
// /////////////////////////////////////////////////////////////////////////////


tlm::tlm_extension_base *data_extension_t::clone() const
{
    tlm::tlm_extension_base * t = new data_extension_t;
    return t;
}

void data_extension_t::copy_from(const tlm::tlm_extension_base & ext)
{
    label = static_cast<data_extension_t const &>(ext).label;
}

